(function() {
  var DEFAULT_GEN_ID_TOKEN, DEFAULT_ID_SKIP_PATTERN, NodeItem, buildItemPathTree, filter, find, jsdom, output, size, _ref;
  _ref = require('underscore'), filter = _ref.filter, find = _ref.find, size = _ref.size;
  jsdom = require('jsdom');
  DEFAULT_ID_SKIP_PATTERN = /[-_]\w*\d+\w*/;
  DEFAULT_GEN_ID_TOKEN = "-GEN_ID";
  NodeItem = (function() {
    function NodeItem(selector, parent) {
      this.selector = selector;
      this.parent = parent != null ? parent : null;
      this.sources = {};
      this.children = [];
      if (this.parent) {
        parent.children.push(this);
      }
    }
    NodeItem.prototype.add = function(source, selector) {
      var nodeItem, _ref;
      nodeItem = (_ref = find(this.children, function(it) {
        return it.selector === selector;
      })) != null ? _ref : new NodeItem(selector, this);
      nodeItem.sources[source.path] = source;
      return nodeItem;
    };
    return NodeItem;
  })();
  this.computeCssTree = function(sources, idSkipPattern, genIdToken) {
    var data, idMasher, root, rootItem, source, window, _i, _len;
    if (idSkipPattern == null) {
      idSkipPattern = DEFAULT_ID_SKIP_PATTERN;
    }
    if (genIdToken == null) {
      genIdToken = DEFAULT_GEN_ID_TOKEN;
    }
    idMasher = function(id) {
      if (idSkipPattern) {
        return id = id.replace(idSkipPattern, genIdToken);
      }
    };
    rootItem = new NodeItem;
    for (_i = 0, _len = sources.length; _i < _len; _i++) {
      source = sources[_i];
      data = source.read();
      window = jsdom.jsdom(data, null).createWindow();
      root = find(window.document.childNodes, function(node) {
        return node.nodeType === 1;
      });
      buildItemPathTree(source, root, rootItem, idMasher);
    }
    return rootItem;
  };
  buildItemPathTree = function(source, node, parentItem, idMasher) {
    var classNames, current, id, s, selector, _i, _len, _ref;
    if (idMasher == null) {
      idMasher = null;
    }
    selector = node.tagName.toLowerCase();
    id = node.getAttribute('id');
    if (id) {
      if (idMasher) {
        id = idMasher(id);
      }
      selector += "#" + id;
    }
    classNames = node.getAttribute('class');
    if (classNames) {
      selector += ((function() {
        var _i, _len, _ref, _results;
        _ref = classNames.split(/\s+/);
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          s = _ref[_i];
          _results.push(s ? '.' + s : void 0);
        }
        return _results;
      })()).join("");
    }
    current = parentItem.add(source, selector);
    _ref = filter(node.childNodes, function(node) {
      return node.nodeType === 1;
    });
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      node = _ref[_i];
      buildItemPathTree(source, node, current, idMasher);
    }
    return;
  };
  this.output = output = function(write, nodeItem, sources, indent, level, parentRatio) {
    var i, it, note, ratio, _i, _len, _ref, _results;
    if (indent == null) {
      indent = 2;
    }
    if (level == null) {
      level = 0;
    }
    if (parentRatio == null) {
      parentRatio = 0;
    }
    if (nodeItem.selector) {
      note = "";
      ratio = size(nodeItem.sources) / sources.length;
      if (ratio < 1.0 && ratio !== parentRatio) {
        note = " /* in " + (Math.round(ratio * 100)) + "% */";
      }
      write(((function() {
        var _ref, _results;
        _results = [];
        for (i = 0, _ref = indent * level; (0 <= _ref ? i < _ref : i > _ref); (0 <= _ref ? i += 1 : i -= 1)) {
          _results.push(' ');
        }
        return _results;
      })()).join(''));
      write(nodeItem.selector + note);
      write("\n");
    } else {
      level = -1;
    }
    _ref = nodeItem.children;
    _results = [];
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      it = _ref[_i];
      _results.push(output(write, it, sources, indent, level + 1, ratio));
    }
    return _results;
  };
}).call(this);
